import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Box, Checkbox, Chip, Grid, IconButton, List, ListItem, ListItemButton, ListItemDecorator, Sheet, Tooltip, } from "@mui/joy";
import { Drawer } from "@mui/material";
import { useLocalStorage } from "@uidotdev/usehooks";
import { DateTime } from "luxon";
import { useEffect, useState } from "react";
import { useDispatch } from "react-redux";
import { Link } from "react-router-dom";
import { useLocation } from "react-router-dom";
import { nerdHistoryOpenSelector, setNerdHistoryOpen, setNerdHistoryOpenKey, } from "../redux/nerdHistory";
import { useSelector } from "react-redux";
/**
 *
 * @param {NerdHistoryProps} props Props
 * @returns {ReactElement} NerdHistory
 */
export function NerdHistory(props) {
    const { historykey, openKey, anchor = "right", pathesToIgnore } = props;
    const location = useLocation();
    const [history, saveHistory] = useLocalStorage(historykey, []);
    const [selectedItems, setSelectedItems] = useState(new Set());
    const [allSelected, setAllSelected] = useState(false);
    const open = useSelector(nerdHistoryOpenSelector);
    const dispatch = useDispatch();
    useEffect(() => {
        saveHistory((history) => {
            const test = history.filter((h) => {
                const date = DateTime.fromMillis(h.date);
                if (DateTime.now().diff(date, "days").days >= 7)
                    return false;
                else
                    return true;
            });
            return test;
        });
    }, []);
    useEffect(() => {
        dispatch(setNerdHistoryOpenKey(openKey));
    }, [openKey]);
    useEffect(() => {
        const _open = localStorage.getItem(openKey);
        dispatch(setNerdHistoryOpen(_open == "true" ? true : false));
    }, []);
    useEffect(() => {
        let match = false;
        if (pathesToIgnore)
            for (let i = 0; i < pathesToIgnore.length; i++) {
                const rx = pathesToIgnore[i];
                match = rx.test(location.pathname);
                if (match)
                    break;
            }
        if (!match)
            setTimeout(() => {
                const index = history.findIndex((h) => h.endpoint === location.pathname);
                if (index !== -1) {
                    const newHistory = [...history];
                    newHistory[index] = {
                        date: DateTime.now().toMillis(),
                        endpoint: location.pathname,
                        label: document.title,
                    };
                    saveHistory(newHistory);
                }
                else
                    saveHistory([
                        ...history,
                        {
                            date: DateTime.now().toMillis(),
                            endpoint: location.pathname,
                            label: document.title,
                        },
                    ]);
            }, 100);
    }, [location]);
    const onClose = () => {
        dispatch(setNerdHistoryOpen(false));
    };
    return (_jsx(_Fragment, { children: _jsx(Drawer, { open: open, onClose: onClose, variant: "persistent", sx: {
                width: 250,
                flexShrink: 0,
                "& .MuiDrawer-paper": {
                    width: 250,
                },
            }, anchor: anchor, children: _jsxs(Box, { sx: {
                    display: "flex",
                    gap: 1,
                    alignItems: "stretch",
                    flexDirection: "column",
                    mt: { xs: "138px", sm: "98px" },
                }, children: [_jsx(Sheet, { sx: {
                            p: 0.5,
                        }, children: _jsxs(Grid, { container: true, spacing: 1, alignItems: "start", children: [_jsx(Grid, { children: _jsx(Chip, { disabled: history.length === 0, size: "sm", variant: "soft", color: "primary", onClick: () => {
                                            if (!allSelected) {
                                                setSelectedItems(new Set(history.map((h) => h.date)));
                                            }
                                            else
                                                setSelectedItems(new Set());
                                            setAllSelected((s) => !s);
                                        }, children: allSelected ? "Unselect All" : "Select All" }) }), _jsx(Grid, { ml: "auto", children: _jsx(Tooltip, { title: "Delete", children: _jsx(IconButton, { disabled: selectedItems.size === 0, size: "sm", variant: "soft", onClick: () => {
                                                saveHistory(history.filter((h) => !selectedItems.has(h.date)));
                                                setSelectedItems(new Set());
                                                setAllSelected(false);
                                            }, children: _jsx("i", { className: "fa-solid fa-trash" }) }) }) }), _jsx(Grid, { children: _jsx(Tooltip, { title: "Close", children: _jsx(IconButton, { size: "sm", variant: "plain", onClick: onClose, children: _jsx("i", { className: "fa-solid fa-xmark" }) }) }) })] }) }), _jsx(Box, { p: 1, children: _jsx(List, { size: "sm", children: history
                                .sort((a, b) => b.date - a.date)
                                .map((h, i) => (_jsxs(ListItem, { children: [_jsx(ListItemDecorator, { children: _jsx(Checkbox, { size: "sm", checked: selectedItems.has(h.date) || false, onChange: (e) => {
                                                const checked = e.target.checked;
                                                setSelectedItems((state) => {
                                                    const newState = new Set(state);
                                                    if (checked)
                                                        newState.add(h.date);
                                                    else
                                                        newState.delete(h.date);
                                                    return newState;
                                                });
                                                if (!checked) {
                                                    setAllSelected(false);
                                                }
                                            } }) }), _jsx(ListItemButton, { component: Link, to: h.endpoint, sx: {
                                            textDecoration: "none !important",
                                        }, children: h.label })] }, i))) }) })] }) }) }));
}
